package org.artifactory.schedule;

import org.artifactory.addon.ArtifactoryRunningMode;
import org.artifactory.state.ArtifactoryServerState;
import org.artifactory.storage.db.servers.model.ArtifactoryServer;
import org.artifactory.storage.db.servers.model.ArtifactoryServerRole;
import org.artifactory.storage.db.servers.service.ArtifactoryServersCommonService;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.when;

/**
 * @author Omri Ziv
 */
public class TaskAffinityServiceImplTest {

    @Mock
    private ArtifactoryServersCommonService artifactoryServersCommonService;

    @BeforeMethod
    public void setUp() {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void testAffinityAnyWithJobName() {
        TaskAffinityService taskAffinityService = new TaskAffinityServiceImpl(artifactoryServersCommonService);
        ArtifactoryServer server1 =
                new ArtifactoryServer("art1", System.currentTimeMillis(), "", 333, ArtifactoryServerState.RUNNING,
                        ArtifactoryServerRole.TASK_AFFINITY,
                        System.currentTimeMillis(), "7.6.0", 1234, 1234, ArtifactoryRunningMode.HA, "aaaa");

        ArtifactoryServer server2 =
                new ArtifactoryServer("art2", System.currentTimeMillis(), "", 333, ArtifactoryServerState.RUNNING,
                        ArtifactoryServerRole.MEMBER,
                        System.currentTimeMillis(), "7.6.0", 1234, 1234, ArtifactoryRunningMode.HA, "aaaa");

        when(artifactoryServersCommonService.getAliveAffinityNodes()).thenReturn(List.of(server1, server2));

        List<String> nodes = taskAffinityService.getAffinityForTask(JobName.CALL_HOME);
        assertEquals(1, nodes.size());
        assertEquals("art1", nodes.get(0));
    }


    @Test
    public void testAffinityAnyWithJobNameMultipleNodes() {
        TaskAffinityService taskAffinityService = new TaskAffinityServiceImpl(artifactoryServersCommonService);
        ArtifactoryServer server1 =
                new ArtifactoryServer("art1", System.currentTimeMillis(), "", 333, ArtifactoryServerState.RUNNING,
                        ArtifactoryServerRole.TASK_AFFINITY,
                        System.currentTimeMillis(), "7.6.0", 1234, 1234, ArtifactoryRunningMode.HA, "aaaa");

        ArtifactoryServer server2 =
                new ArtifactoryServer("art2", System.currentTimeMillis(), "", 333, ArtifactoryServerState.RUNNING,
                        ArtifactoryServerRole.TASK_AFFINITY,
                        System.currentTimeMillis(), "7.6.0", 1234, 1234, ArtifactoryRunningMode.HA, "aaaa");

        ArtifactoryServer server3 =
                new ArtifactoryServer("art3", System.currentTimeMillis(), "", 333, ArtifactoryServerState.RUNNING,
                        ArtifactoryServerRole.TASK_AFFINITY,
                        System.currentTimeMillis(), "7.6.0", 1234, 1234, ArtifactoryRunningMode.HA, "aaaa");

        when(artifactoryServersCommonService.getAliveAffinityNodes()).thenReturn(List.of(server1, server2, server3));

        List<String> nodes = taskAffinityService.getAffinityForTask(JobName.CALL_HOME);
        assertEquals(3, nodes.size());
        assertTrue(nodes.containsAll(List.of(server1.getServerId(), server2.getServerId(), server3.getServerId())));
    }

    @Test
    public void testAffinityAnyAndNoneWithJobNameMultipleNodes() {
        TaskAffinityService taskAffinityService = new TaskAffinityServiceImpl(artifactoryServersCommonService);
        ArtifactoryServer server1 =
                new ArtifactoryServer("art1", System.currentTimeMillis(), "", 333, ArtifactoryServerState.RUNNING,
                        ArtifactoryServerRole.TASK_AFFINITY,
                        System.currentTimeMillis(), "7.6.0", 1234, 1234, ArtifactoryRunningMode.HA, "aaaa");

        ArtifactoryServer server2 =
                new ArtifactoryServer("art2", System.currentTimeMillis(), "", 333, ArtifactoryServerState.RUNNING,
                        ArtifactoryServerRole.TASK_AFFINITY,
                        System.currentTimeMillis(), "7.6.0", 1234, 1234, ArtifactoryRunningMode.HA, "aaaa");

        ArtifactoryServer server3 =
                new ArtifactoryServer("art3", System.currentTimeMillis(), "", 333, ArtifactoryServerState.RUNNING,
                        ArtifactoryServerRole.MEMBER,
                        System.currentTimeMillis(), "7.6.0", 1234, 1234, ArtifactoryRunningMode.HA, "aaaa");

        ArtifactoryServer server4 =
                new ArtifactoryServer("art4", System.currentTimeMillis(), "", 333, ArtifactoryServerState.RUNNING,
                        ArtifactoryServerRole.MEMBER,
                        System.currentTimeMillis(), "7.6.0", 1234, 1234, ArtifactoryRunningMode.HA, "aaaa");

        when(artifactoryServersCommonService.getAliveAffinityNodes())
                .thenReturn(List.of(server1, server2, server3, server4));

        List<String> nodes = taskAffinityService.getAffinityForTask(JobName.CALL_HOME);
        assertEquals(2, nodes.size());
        assertTrue(nodes.containsAll(List.of(server1.getServerId(), server2.getServerId())));
    }

    @Test
    public void testAffinityNoneWithJobNameMultipleNodes() {
        TaskAffinityService taskAffinityService = new TaskAffinityServiceImpl(artifactoryServersCommonService);
        ArtifactoryServer server1 =
                new ArtifactoryServer("art1", System.currentTimeMillis(), "", 333, ArtifactoryServerState.RUNNING,
                        ArtifactoryServerRole.MEMBER,
                        System.currentTimeMillis(), "7.6.0", 1234, 1234, ArtifactoryRunningMode.HA, "aaaa");

        ArtifactoryServer server2 =
                new ArtifactoryServer("art2", System.currentTimeMillis(), "", 333, ArtifactoryServerState.RUNNING,
                        ArtifactoryServerRole.MEMBER,
                        System.currentTimeMillis(), "7.6.0", 1234, 1234, ArtifactoryRunningMode.HA, "aaaa");

        when(artifactoryServersCommonService.getAliveAffinityNodes()).thenReturn(List.of(server1, server2));

        List<String> nodes = taskAffinityService.getAffinityForTask(JobName.CALL_HOME);
        assertEquals(0, nodes.size());
    }
}
