package org.artifactory.storage.db.conversion.version.v225;

import org.apache.commons.lang.StringUtils;
import org.artifactory.storage.db.version.ConversionPredicate;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;
import org.jfrog.storage.util.DbUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.SQLException;
import java.util.function.BiPredicate;

/**
 * @author Alexei Vainshtein
 */
public class V225ConversionPredicate implements ConversionPredicate {
    private static final Logger log = LoggerFactory.getLogger(V225ConversionPredicate.class);

    private final String indexName;
    private final String[] columnsToCheck;

    public V225ConversionPredicate(String indexName, String... columnsToCheck) {
        this.indexName = indexName;
        this.columnsToCheck = columnsToCheck;
    }

    @Override
    public BiPredicate<JdbcHelper, DbType> condition() {
        return (jdbcHelper, dbType) -> {
            if (DbType.POSTGRESQL != dbType) {
                return false;
            }

            // Check in the db if the index is the right one.
            try {
                String indexDef =
                        DbUtils.getPostgresqlIndexDefInfo(jdbcHelper, indexName);
                if (StringUtils.isBlank(indexDef)) {
                    log.debug("Index {} not found, converter need to run.", indexDef);
                    return true;
                }
                for (String columnToCheck : columnsToCheck) {
                    if (!indexDef.contains(columnToCheck)) {
                        // Reached here means the index is not complete. Run the convert.
                        return true;
                    }
                }
                // Reached here, means the info exists, don't run the convert again.
                return false;
            } catch (SQLException e) {
                log.error("An error occurred while querying the database: {}", e.getMessage());
                throw new RuntimeException("An error occurred while querying the database", e);
            }
        };
    }
}
