package org.artifactory.storage.db.upgrades.common;

import org.artifactory.aol.AolService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.opentracing.ArtifactoryTracer;
import org.artifactory.storage.db.init.CommonDataSourceFactory;
import org.artifactory.storage.db.util.JdbcHelper;
import org.mockito.Mockito;
import org.testng.Assert;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;

import java.util.concurrent.TimeUnit;

import static org.artifactory.storage.db.version.ArtifactoryDBVersion.v100;

/**
 * @author Alexei Vainshtein
 */
public abstract class AsyncUpgradeBaseTest extends UpgradeBaseTest {

    private DatabaseTestConverterAdapter asyncDatabaseConverterAdapter;

    @BeforeClass
    @Override
    protected void springTestContextPrepareTestInstance() throws Exception {
        super.springTestContextPrepareTestInstance();
        asyncDatabaseConverterAdapter = new DatabaseTestConverterAdapter(ArtifactoryHome.get());
        runAsyncConversion();
        waitForAsyncToFinish();
    }

    protected void runAsyncConversion() {
        ArtifactoryContext context = Mockito.mock(ArtifactoryContext.class);
        ArtifactoryTracer tracer = Mockito.mock(ArtifactoryTracer.class);
        Mockito.when(tracer.getCurrentThreadActiveSpan()).thenReturn(null);
        Mockito.when(context.beanForType(ArtifactoryTracer.class)).thenReturn(tracer);
        AolService aolService = Mockito.mock(AolService.class);
        Mockito.when(aolService.isAol()).thenReturn(false);
        Mockito.when(context.beanForType(AolService.class)).thenReturn(aolService);
        Mockito.when(context.beanForType(JdbcHelper.class))
                .thenReturn(new org.artifactory.storage.db.util.JdbcHelper(
                        CommonDataSourceFactory
                                .createDataSource(dbProperties,
                                        false)));
        Mockito.when(context.getArtifactoryHome()).thenReturn(ArtifactoryHome.get());
        ArtifactoryContextThreadBinder.bind(context);
        asyncDatabaseConverterAdapter.convert(toCompoundVersionDetails(v100), null);
    }

    @AfterClass
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    // Waits for 5 seconds to the async to finish.
    // If not finished after 5 seconds, fail the test
    protected void waitForAsyncToFinish() throws InterruptedException {
        if (!asyncDatabaseConverterAdapter.getScheduledExecutorService().awaitTermination(5, TimeUnit.SECONDS)) {
            Assert.fail("Executor wasn't terminated after 5 seconds. Failing the test.");
        }
    }
}
